"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const fetch_1 = __importDefault(require("./fetch"));
const assert_1 = require("assert");
const OIDC_DISCOVERY = '/.well-known/openid-configuration';
const OAUTH2_DISCOVERY = '/.well-known/oauth-authorization-server';
const assertIssuer = (data) => (0, assert_1.strict)(data.issuer, `'issuer' not found in authorization server metadata`);
const discover = async (uri, { agent, timeoutDuration } = {}) => {
    const url = new url_1.URL(uri);
    if (url.pathname.includes('/.well-known/')) {
        const data = await (0, fetch_1.default)(url, { agent, timeoutDuration });
        assertIssuer(data);
        return data;
    }
    const pathnames = [];
    if (url.pathname.endsWith('/')) {
        pathnames.push(`${url.pathname}${OIDC_DISCOVERY.substring(1)}`);
    }
    else {
        pathnames.push(`${url.pathname}${OIDC_DISCOVERY}`);
    }
    if (url.pathname === '/') {
        pathnames.push(`${OAUTH2_DISCOVERY}`);
    }
    else {
        pathnames.push(`${OAUTH2_DISCOVERY}${url.pathname}`);
    }
    for (const pathname of pathnames) {
        try {
            const wellKnownUri = new url_1.URL(pathname, url);
            const data = await (0, fetch_1.default)(wellKnownUri, {
                agent,
                timeoutDuration,
            });
            assertIssuer(data);
            return data;
        }
        catch (err) {
        }
    }
    throw new Error('Failed to fetch authorization server metadata');
};
exports.default = discover;
