"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = require("assert");
const buffer_1 = require("buffer");
const crypto_1 = require("crypto");
const url_1 = require("url");
const jose_1 = require("jose");
const oauth2_bearer_1 = require("./oauth2-bearer");
const discovery_1 = __importDefault(require("./discovery"));
const validate_1 = __importStar(require("./validate"));
const ASYMMETRIC_ALGS = [
    'RS256',
    'RS384',
    'RS512',
    'PS256',
    'PS384',
    'PS512',
    'ES256',
    'ES256K',
    'ES384',
    'ES512',
    'EdDSA',
];
const SYMMETRIC_ALGS = ['HS256', 'HS384', 'HS512'];
const jwtVerifier = ({ issuerBaseURL = process.env.ISSUER_BASE_URL, jwksUri = process.env.JWKS_URI, issuer = process.env.ISSUER, audience = process.env.AUDIENCE, secret = process.env.SECRET, tokenSigningAlg = process.env.TOKEN_SIGNING_ALG, agent, cooldownDuration = 30000, timeoutDuration = 5000, clockTolerance = 5, maxTokenAge, strict = false, validators: customValidators, }) => {
    let origJWKS;
    let discovery;
    let validators;
    let allowedSigningAlgs;
    (0, assert_1.strict)(issuerBaseURL || (issuer && jwksUri) || (issuer && secret), "You must provide an 'issuerBaseURL', an 'issuer' and 'jwksUri' or an 'issuer' and 'secret'");
    (0, assert_1.strict)(!(secret && jwksUri), "You must not provide both a 'secret' and 'jwksUri'");
    (0, assert_1.strict)(audience, "An 'audience' is required to validate the 'aud' claim");
    (0, assert_1.strict)(!secret || (secret && tokenSigningAlg), "You must provide a 'tokenSigningAlg' for validating symmetric algorithms");
    (0, assert_1.strict)(secret || !tokenSigningAlg || ASYMMETRIC_ALGS.includes(tokenSigningAlg), `You must supply one of ${ASYMMETRIC_ALGS.join(', ')} for 'tokenSigningAlg' to validate asymmetrically signed tokens`);
    (0, assert_1.strict)(!secret || (tokenSigningAlg && SYMMETRIC_ALGS.includes(tokenSigningAlg)), `You must supply one of ${SYMMETRIC_ALGS.join(', ')} for 'tokenSigningAlg' to validate symmetrically signed tokens`);
    const secretKey = secret && (0, crypto_1.createSecretKey)(buffer_1.Buffer.from(secret));
    const JWKS = async (...args) => {
        if (secretKey)
            return secretKey;
        if (!origJWKS) {
            origJWKS = (0, jose_1.createRemoteJWKSet)(new url_1.URL(jwksUri), {
                agent,
                cooldownDuration,
                timeoutDuration,
            });
        }
        return origJWKS(...args);
    };
    return async (jwt) => {
        try {
            if (issuerBaseURL) {
                discovery =
                    discovery || (0, discovery_1.default)(issuerBaseURL, { agent, timeoutDuration });
                ({
                    jwks_uri: jwksUri,
                    issuer,
                    id_token_signing_alg_values_supported: allowedSigningAlgs,
                } = await discovery);
            }
            validators || (validators = {
                ...(0, validate_1.defaultValidators)(issuer, audience, clockTolerance, maxTokenAge, strict, allowedSigningAlgs, tokenSigningAlg),
                ...customValidators,
            });
            const { payload, protectedHeader: header } = await (0, jose_1.jwtVerify)(jwt, JWKS);
            await (0, validate_1.default)(payload, header, validators);
            return { payload, header, token: jwt };
        }
        catch (e) {
            throw new oauth2_bearer_1.InvalidTokenError(e.message);
        }
    };
};
exports.default = jwtVerifier;
